clc;                             % clear command window/console
clear;                           % clear workspace                                      
close all;                       % close all windows (plots)

import casadi.*

% create empty optimization problem
opti = casadi.Opti();

N = 40;

y = opti.variable(N,1);
z = opti.variable(N,1);

mi = 4/N;
Di = (70/40)*N;
g0 = 9.81;

% defining the objective
Vchain = 0;
% gravitational terms
for i = 1:N
   Vchain = Vchain + g0*mi*z(i);
end
% potential energy of the springs
for i = 1:N-1
    Vchain = Vchain + 1/2*Di*((y(i)-y(i+1))^2 + (z(i)-z(i+1))^2);
end

% passing the objective to opti
opti.minimize(Vchain)

opti.subject_to( y(1) == -2 )
opti.subject_to( z(1) ==  1 )
opti.subject_to( y(N) ==  2 )
opti.subject_to( z(N) ==  1 )

% Setting solver to ipopt and solving the problem:
opti.solver('ipopt')
% opti.solver('sqpmethod')
sol = opti.solve();

Y = sol.value(y);
Z = sol.value(z);

figure;
plot(Y,Z,'--or'); hold on;
plot(-2,1,'xg','MarkerSize',10);
plot(2,1,'xg','MarkerSize',10);
xlabel('y'); ylabel('z');
title('Optimal solution hanging chain (without extra constraints)')

%% adding the ground constraints
opti.subject_to(       z >= 0.5);
opti.subject_to( z-0.1*y >= 0.5); 

sol2 = opti.solve();

Y2 = sol2.value(y);
Z2 = sol2.value(z);

figure;
plot(Y2,Z2,'--or'); hold on;
plot(-2,1,'xg','MarkerSize',10);
plot(2,1,'xg','MarkerSize',10);
plot([-2 2],[0.5 0.5],':b');
plot([-2 2],[0.3 0.7],':b');
xlabel('y'); ylabel('z');
title('Optimal solution hanging chain (with linear ground constraints)')
